import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import '../features/call/call_page.dart';
import '../features/call/incoming_call_page.dart';

class CallService {
  static const MethodChannel _channel = MethodChannel('call_service');
  static String? currentPhoneNumber;
  static bool _isCallPageVisible = false;
  static Map<String, dynamic>? _pendingCall;
  static bool wasPhoneLocked = false;

  static final GlobalKey<NavigatorState> navigatorKey = GlobalKey<NavigatorState>();

  CallService() {
    _channel.setMethodCallHandler((call) async {
      print('CallService: Handling method call: ${call.method}');
      switch (call.method) {
        case "callAdded":
          final phoneNumber = call.arguments["callId"] as String;
          final state = call.arguments["state"] as String;
          currentPhoneNumber = phoneNumber.replaceFirst('tel:', '');
          print('CallService: Call added, number: $currentPhoneNumber, state: $state');
          if (state == "ringing") {
            _handleIncomingCall(phoneNumber);
          } else {
            _navigateToCallPage();
          }
          break;
        case "callStateChanged":
          final state = call.arguments["state"] as String;
          wasPhoneLocked = call.arguments["wasPhoneLocked"] as bool? ?? false;
          print('CallService: State changed to $state, wasPhoneLocked: $wasPhoneLocked');
          if (state == "disconnected" || state == "disconnecting") {
            _closeCallPage();
            if (wasPhoneLocked) {
              _channel.invokeMethod("callEndedFromFlutter");
            }
          } else if (state == "active" || state == "dialing") {
            _navigateToCallPage();
          } else if (state == "ringing") {
            final phoneNumber = call.arguments["callId"] as String;
            _handleIncomingCall(phoneNumber.replaceFirst('tel:', ''));
          }
          break;
        case "callEnded":
        case "callRemoved":
          wasPhoneLocked = call.arguments["wasPhoneLocked"] as bool? ?? false;
          print('CallService: Call ended/removed, wasPhoneLocked: $wasPhoneLocked');
          _closeCallPage();
          if (wasPhoneLocked) {
            _channel.invokeMethod("callEndedFromFlutter");
          }
          currentPhoneNumber = null;
          break;
        case "incomingCallFromNotification":
          final phoneNumber = call.arguments["phoneNumber"] as String;
          wasPhoneLocked = call.arguments["wasPhoneLocked"] as bool? ?? false;
          currentPhoneNumber = phoneNumber;
          print('CallService: Incoming call from notification: $phoneNumber, wasPhoneLocked: $wasPhoneLocked');
          _handleIncomingCall(phoneNumber);
          break;
      }
    });
  }

  void _handleIncomingCall(String phoneNumber) {
    final context = navigatorKey.currentContext;
    if (context == null) {
      print('CallService: Context is null, queuing incoming call: $phoneNumber');
      _pendingCall = {"phoneNumber": phoneNumber};
      Future.delayed(Duration(milliseconds: 500), () => _checkPendingCall());
    } else {
      _navigateToIncomingCallPage(context);
    }
  }

  void _checkPendingCall() {
    if (_pendingCall != null) {
      final context = navigatorKey.currentContext;
      if (context != null) {
        print('CallService: Processing queued call: ${_pendingCall!["phoneNumber"]}');
        currentPhoneNumber = _pendingCall!["phoneNumber"];
        _navigateToIncomingCallPage(context);
        _pendingCall = null;
      } else {
        print('CallService: Context still null, retrying...');
        Future.delayed(Duration(milliseconds: 500), () => _checkPendingCall());
      }
    }
  }

  void _navigateToCallPage() {
    final context = navigatorKey.currentContext;
    if (context == null) {
      print('CallService: Cannot navigate to CallPage, context is null');
      return;
    }
    if (_isCallPageVisible) {
      print('CallService: CallPage already visible, skipping navigation');
      return;
    }
    print('CallService: Navigating to CallPage');
    Navigator.push(
      context,
      MaterialPageRoute(
        settings: const RouteSettings(name: '/call'),
        builder: (context) => CallPage(
          displayName: currentPhoneNumber!,
          phoneNumber: currentPhoneNumber!,
          thumbnail: null,
        ),
      ),
    ).then((_) {
      _isCallPageVisible = false;
      print('CallService: CallPage popped, _isCallPageVisible set to false');
    });
    _isCallPageVisible = true;
  }

  void _navigateToIncomingCallPage(BuildContext context) {
    if (_isCallPageVisible) {
      print('CallService: IncomingCallPage already visible, skipping navigation');
      return;
    }
    print('CallService: Navigating to IncomingCallPage');
    Navigator.push(
      context,
      MaterialPageRoute(
        settings: const RouteSettings(name: '/incoming_call'),
        builder: (context) => IncomingCallPage(
          displayName: currentPhoneNumber!,
          phoneNumber: currentPhoneNumber!,
          thumbnail: null,
        ),
      ),
    ).then((_) {
      _isCallPageVisible = false;
      print('CallService: IncomingCallPage popped, _isCallPageVisible set to false');
    });
    _isCallPageVisible = true;
  }

  void _closeCallPage() {
    final context = navigatorKey.currentContext;
    if (context == null) {
      print('CallService: Cannot close page, context is null');
      return;
    }
    print('CallService: Closing call page, _isCallPageVisible: $_isCallPageVisible');
    if (Navigator.canPop(context)) {
      print('CallService: Popping call page');
      Navigator.pop(context);
      _isCallPageVisible = false;
    } else {
      print('CallService: No page to pop');
    }
  }

  Future<Map<String, dynamic>> makeGsmCall(
    BuildContext context, {
    required String phoneNumber,
    String? displayName,
    Uint8List? thumbnail,
  }) async {
    try {
      currentPhoneNumber = phoneNumber;
      print('CallService: Making GSM call to $phoneNumber');
      final result = await _channel.invokeMethod('makeGsmCall', {"phoneNumber": phoneNumber});
      print('CallService: makeGsmCall result: $result');
      final resultMap = Map<String, dynamic>.from(result as Map);
      if (resultMap["status"] != "calling") {
        ScaffoldMessenger.of(context).showSnackBar(
          SnackBar(content: Text("Failed to initiate call")),
        );
      }
      return resultMap;
    } catch (e) {
      print("CallService: Error making call: $e");
      ScaffoldMessenger.of(context).showSnackBar(
        SnackBar(content: Text("Error making call: $e")),
      );
      return {"status": "error", "message": e.toString()};
    }
  }

  Future<Map<String, dynamic>> hangUpCall(BuildContext context) async {
    try {
      print('CallService: Hanging up call');
      final result = await _channel.invokeMethod('hangUpCall');
      print('CallService: hangUpCall result: $result');
      final resultMap = Map<String, dynamic>.from(result as Map);
      if (resultMap["status"] != "ended") {
        ScaffoldMessenger.of(context).showSnackBar(
          SnackBar(content: Text("Failed to end call")),
        );
      }
      return resultMap;
    } catch (e) {
      print("CallService: Error hanging up call: $e");
      ScaffoldMessenger.of(context).showSnackBar(
        SnackBar(content: Text("Error hanging up call: $e")),
      );
      return {"status": "error", "message": e.toString()};
    }
  }
}